@@ -98,6 +98,11 @@ enum {
 	DEVICE_TYPE_FILTER_NUM_COLS
 };
 
+enum {
+	TREEVIEW_COLUMN_DEVICE = 0,
+	TREEVIEW_COLUMN_TYPE = 1
+};
+
 static void
 bonded_to_icon (GtkTreeViewColumn *column, GtkCellRenderer *cell,
 	      GtkTreeModel *model, GtkTreeIter *iter, gpointer data)
@@ -194,8 +199,8 @@ bluetooth_chooser_start_discovery (BluetoothChooser *self)
 
 	g_return_if_fail (priv->show_searching);
 
-	if (bluetooth_client_start_discovery (priv->client) != FALSE)
-		set_search_label (self, TRUE);
+	g_object_set (G_OBJECT (priv->client), "default-adapter-discovering", TRUE, NULL);
+	set_search_label (self, TRUE);
 	priv->disco_rq = TRUE;
 }
 
@@ -213,7 +218,7 @@ bluetooth_chooser_stop_discovery (BluetoothChooser *self)
 	g_return_if_fail (priv->show_searching);
 
 	priv->disco_rq = FALSE;
-	bluetooth_client_stop_discovery (priv->client);
+	g_object_set (G_OBJECT (priv->client), "default-adapter-discovering", FALSE, NULL);
 }
 
 static char *
@@ -362,78 +367,23 @@ bluetooth_chooser_get_selected_device_info (BluetoothChooser *self,
 	return TRUE;
 }
 
-static gboolean
-show_confirm_dialog (BluetoothChooser *chooser, const char *name)
-{
-	GtkWidget *dialog, *parent;
-	gint response;
-
-	parent = gtk_widget_get_toplevel (GTK_WIDGET (chooser));
-	dialog = gtk_message_dialog_new (GTK_WINDOW (parent), GTK_DIALOG_MODAL,
-					 GTK_MESSAGE_QUESTION, GTK_BUTTONS_NONE,
-					 _("Remove '%s' from the list of devices?"), name);
-	g_object_set (G_OBJECT (dialog), "secondary-text",
-		      _("If you remove the device, you will have to set it up again before next use."),
-		      NULL);
-
-	gtk_dialog_add_button (GTK_DIALOG (dialog), GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
-	gtk_dialog_add_button (GTK_DIALOG (dialog), GTK_STOCK_REMOVE, GTK_RESPONSE_ACCEPT);
-
-	response = gtk_dialog_run (GTK_DIALOG (dialog));
-
-	gtk_widget_destroy (dialog);
-
-	if (response == GTK_RESPONSE_ACCEPT)
-		return TRUE;
-
-	return FALSE;
-}
-
 /**
- * bluetooth_chooser_remove_selected_device:
+ * bluetooth_chooser_dump_selected_device:
  * @self: A #BluetoothChooser widget.
  *
- * Returns whether the removal was successful.
- *
- * Return value: %TRUE if the selected device was correctly removed.
+ * Prints all the known attributes for the currently selected device
+ * on the standard output. Useful for debugging.
  **/
-gboolean
-bluetooth_chooser_remove_selected_device (BluetoothChooser *self)
+void
+bluetooth_chooser_dump_selected_device (BluetoothChooser *self)
 {
 	BluetoothChooserPrivate *priv = BLUETOOTH_CHOOSER_GET_PRIVATE(self);
 	GtkTreeIter iter;
-	gboolean ret = FALSE;
-	GError *err = NULL;
-	gchar *name;
-	DBusGProxy *device, *adapter;
+	GtkTreeModel *model;
 
-	if (gtk_tree_selection_get_selected (priv->selection, NULL, &iter) == FALSE)
-		return FALSE;
+	gtk_tree_selection_get_selected (priv->selection, &model, &iter);
 
-	gtk_tree_model_get (priv->filter, &iter,
-			    BLUETOOTH_COLUMN_PROXY, &device,
-			    BLUETOOTH_COLUMN_ALIAS, &name, -1);
-
-	adapter = bluetooth_client_get_default_adapter(priv->client);
-
-	if (show_confirm_dialog (self, name) != FALSE) {
-		const gchar *device_path;
-
-		device_path = dbus_g_proxy_get_path (device);
-
-		if (dbus_g_proxy_call (adapter, "RemoveDevice", &err,
-				       DBUS_TYPE_G_OBJECT_PATH, device_path,
-				       G_TYPE_INVALID, G_TYPE_INVALID) == FALSE) {
-			g_warning ("Failed to remove device %s: %s", name,
-				   err->message);
-			g_error_free (err);
-		} else {
-			ret = TRUE;
-		}
-		g_object_unref (adapter);
-	}
-
-	return ret;
+	bluetooth_client_dump_device (model, &iter);
 }
 
 /**
@@ -452,38 +402,6 @@ bluetooth_chooser_get_model (BluetoothChooser *self)
 	return priv->model;
 }
 
-/**
- * bluetooth_chooser_get_device_column:
- * @self: A #BluetoothChooser widget.
- *
- * Returns a #GtkTreeViewColumn object to the device column of the #BluetoothChooser.
- *
- * Return value: a #GtkTreeViewColumn object.
- **/
-GtkTreeViewColumn *
-bluetooth_chooser_get_device_column (BluetoothChooser *self)
-{
-	BluetoothChooserPrivate *priv = BLUETOOTH_CHOOSER_GET_PRIVATE(self);
-
-	return gtk_tree_view_get_column (GTK_TREE_VIEW (priv->treeview), 0);
-}
-
-/**
- * bluetooth_chooser_get_type_column:
- * @self: A #BluetoothChooser widget.
- *
- * Returns a #GtkTreeViewColumn object to the type column of the #BluetoothChooser.
- *
- * Return value: a #GtkTreeViewColumn object.
- **/
-GtkTreeViewColumn *
-bluetooth_chooser_get_type_column (BluetoothChooser *self)
-{
-	BluetoothChooserPrivate *priv = BLUETOOTH_CHOOSER_GET_PRIVATE(self);
-
-	return gtk_tree_view_get_column (GTK_TREE_VIEW (priv->treeview), 1);
-}
-
 /**
  * bluetooth_chooser_get_treeview:
  * @self: A #BluetoothChooser widget.
@@ -505,8 +423,11 @@ bluetooth_chooser_get_treeview (BluetoothChooser *self)
  * @self: A #BluetoothChooser widget.
  *
  * Returns the #GtkScrolledWindow object for the #BluetoothChooser.
+ * This is useful to set a minimum height to the chooser using
+ * gtk_scrolled_window_set_min_content_height() or to join the
+ * chooser with a toolbar.
  *
- * Return value: a #GtkScrolledWindow object.
+ * Return value: (transfer none): a #GtkScrolledWindow object.
  **/
 GtkWidget *
 bluetooth_chooser_get_scrolled_window (BluetoothChooser *self)
@@ -714,7 +635,7 @@ adapter_model_row_changed (GtkTreeModel *model,
 	if (is_default == FALSE)
 		return;
 	if (powered != FALSE && discovering == FALSE && priv->disco_rq != FALSE) {
-		bluetooth_chooser_start_discovery (self);
+		g_object_set (G_OBJECT (priv->client), "default-adapter-discovering", TRUE, NULL);
 		set_search_label (self, TRUE);
 		return;
 	}
@@ -748,7 +669,7 @@ static void default_adapter_changed (GObject    *gobject,
 
 	g_free (adapter);
 
-	priv->model = bluetooth_client_get_device_model (priv->client, NULL);
+	priv->model = bluetooth_client_get_device_model (priv->client);
 	if (priv->model) {
 		priv->filter = gtk_tree_model_filter_new (priv->model, NULL);
 		gtk_tree_model_filter_set_visible_func (GTK_TREE_MODEL_FILTER (priv->filter),
@@ -844,7 +765,7 @@ create_treeview (BluetoothChooser *self)
 			  G_CALLBACK(select_browse_device_callback), self);
 
 	/* Set the model, and filter */
-	priv->model = bluetooth_client_get_device_model (priv->client, NULL);
+	priv->model = bluetooth_client_get_device_model (priv->client);
 	if (priv->model) {
 		priv->filter = gtk_tree_model_filter_new (priv->model, NULL);
 		gtk_tree_model_filter_set_visible_func (GTK_TREE_MODEL_FILTER (priv->filter),
@@ -882,7 +803,7 @@ bluetooth_chooser_init(BluetoothChooser *self)
 	gtk_box_set_spacing (GTK_BOX(self), 18);
 	gtk_container_set_border_width (GTK_CONTAINER(self), 0);
 
-	vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
+	vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
 	gtk_widget_show (vbox);
 	gtk_box_pack_start (GTK_BOX (self), vbox, TRUE, TRUE, 0);
 
@@ -907,7 +828,7 @@ bluetooth_chooser_init(BluetoothChooser *self)
 	gtk_widget_set_name (priv->search_hbox, "search_hbox");
 	if (priv->show_searching)
 		gtk_widget_show (priv->search_hbox);
-	gtk_box_pack_start (GTK_BOX (vbox), priv->search_hbox, FALSE, TRUE, 0);
+	gtk_box_pack_end (GTK_BOX (vbox), priv->search_hbox, FALSE, TRUE, 0);
 	gtk_widget_set_no_show_all (priv->search_hbox, TRUE);
 
 	/* Setup the adapter disco mode callback for the search button */
@@ -926,8 +847,14 @@ bluetooth_chooser_init(BluetoothChooser *self)
 	gtk_widget_show (priv->search_label);
 	gtk_widget_show (hbox);
 
-	gtk_box_pack_end (GTK_BOX (priv->search_hbox), hbox, FALSE, TRUE, 0);
-	//FIXME check whether the default adapter is discovering right now
+	if (priv->show_searching) {
+		gboolean discovering;
+
+		g_object_get (G_OBJECT (priv->client), "default-adapter-discovering", &discovering, NULL);
+		set_search_label (self, discovering);
+	}
+
+	gtk_box_pack_start (GTK_BOX (priv->search_hbox), hbox, FALSE, TRUE, 0);
 
 	/* The treeview */
 	priv->scrolled_window = create_treeview (self);
@@ -989,7 +916,7 @@ bluetooth_chooser_finalize (GObject *object)
 		g_signal_handler_disconnect (G_OBJECT(priv->client), priv->default_adapter_changed_id);
 		priv->default_adapter_changed_id = 0;
 
-		bluetooth_client_stop_discovery (priv->client);
+		g_object_set (G_OBJECT (priv->client), "default-adapter-discovering", FALSE, NULL);
 		g_object_unref (priv->client);
 		priv->client = NULL;
 	}
@@ -1014,6 +941,7 @@ enum {
 	PROP_SHOW_CONNECTED,
 	PROP_SHOW_SEARCHING,
 	PROP_SHOW_DEVICE_TYPE,
+	PROP_SHOW_DEVICE_TYPE_COLUMN,
 	PROP_SHOW_DEVICE_CATEGORY,
 	PROP_DEVICE_TYPE_FILTER,
 	PROP_DEVICE_CATEGORY_FILTER,
@@ -1090,6 +1018,20 @@ bluetooth_chooser_set_property (GObject *object, guint prop_id,
 				g_object_set (G_OBJECT (priv->filters_vbox), "visible", FALSE, NULL);
 		}
 		break;
+	case PROP_SHOW_DEVICE_TYPE_COLUMN: {
+		GtkTreeViewColumn *column;
+
+		column = gtk_tree_view_get_column (GTK_TREE_VIEW (priv->treeview), TREEVIEW_COLUMN_TYPE);
+		gtk_tree_view_column_set_visible (column,
+						  g_value_get_boolean (value));
+
+		column = gtk_tree_view_get_column (GTK_TREE_VIEW (priv->treeview), TREEVIEW_COLUMN_DEVICE);
+		if (g_value_get_boolean (value))
+			gtk_tree_view_column_set_title (column, _("Device"));
+		else
+			gtk_tree_view_column_set_title (column, _("Devices"));
+		break;
+		}
 	case PROP_SHOW_DEVICE_CATEGORY:
 		priv->show_device_category = g_value_get_boolean (value);
 		if (priv->internal_filter) {
@@ -1146,6 +1088,10 @@ bluetooth_chooser_get_property (GObject *object, guint prop_id,
 	case PROP_SHOW_DEVICE_TYPE:
 		g_value_set_boolean (value, priv->show_device_type);
 		break;
+	case PROP_SHOW_DEVICE_TYPE_COLUMN:
+		g_value_set_boolean (value,
+				     gtk_tree_view_column_get_visible (gtk_tree_view_get_column (GTK_TREE_VIEW (priv->treeview), TREEVIEW_COLUMN_TYPE)));
+		break;
 	case PROP_SHOW_DEVICE_CATEGORY:
 		g_value_set_boolean (value, priv->show_device_category);
 		break;
@@ -1238,6 +1184,9 @@ bluetooth_chooser_class_init (BluetoothChooserClass *klass)
 	g_object_class_install_property (G_OBJECT_CLASS(klass),
 					 PROP_SHOW_DEVICE_TYPE, g_param_spec_boolean ("show-device-type",
 										      "show-device-type", "Whether to show the device type filter", TRUE, G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+	g_object_class_install_property (G_OBJECT_CLASS(klass),
+					 PROP_SHOW_DEVICE_TYPE_COLUMN, g_param_spec_boolean ("show-device-type-column",
+											     "show-device-type-column", "Whether to show the device type column", TRUE, G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 	g_object_class_install_property (G_OBJECT_CLASS(klass),
 					 PROP_SHOW_DEVICE_CATEGORY, g_param_spec_boolean ("show-device-category",
 											  "show-device-category", "Whether to show the device category filter", TRUE, G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
